==> 5.1.9.tar.gz: lib/navigation/osam_navigator.dart <==
import 'dart:async';

import 'package:flutter/material.dart';

import '../osam.dart';
import 'navigation_presenter.dart';

PageRoute _pageRoute({WidgetBuilder builder, RouteSettings settings, bool isMaterial = false}) =>
    isMaterial
        ? MaterialPageRoute(builder: builder, settings: settings)
        : CupertinoPageRoute(builder: builder, settings: settings);

class OsamNavigator<P extends NavigationPresenter> extends StatefulWidget {
  final Map<String, WidgetBuilder> _screens;
  final bool rootNavigator;
  final bool isMaterial;

  const OsamNavigator(this._screens, {Key key, this.rootNavigator = false, this.isMaterial = false})
      : super(key: key);

  @override
  _OsamNavigatorState createState() => _OsamNavigatorState<P>();
}

class _OsamNavigatorState<P extends NavigationPresenter> extends State<OsamNavigator>
    with AfterLayoutMixin {
  final List<String> _routes = [];
  StreamSubscription<List<String>> routesSub;

  Map<String, WidgetBuilder> get screens => widget._screens;

  @override
  Widget build(BuildContext context) =>
      screens[context.presenter<P>().routesStream.value.first](context);

  void _silentPop(NavigationPresenter presenter) {
    if (presenter.routesStream.value.length > 1) {
      _routes.removeLast();
      presenter.pop();
    }
  }

  @override
  void afterFirstLayout(BuildContext context) {
    final navigator = Navigator.of(context, rootNavigator: widget.rootNavigator);
    final presenter = context.presenter<P>();
    final initialRoutes = presenter.routesStream.value;
    _routes.addAll(initialRoutes);
    if (initialRoutes.length > 1) {
      final firstRoute = initialRoutes.first;
      navigator.pushReplacement(_pageRoute(
          builder: ((ctx) => screens[firstRoute])(context),
          settings: const RouteSettings(isInitialRoute: true),
          isMaterial: widget.isMaterial));
      final others = initialRoutes.sublist(1, initialRoutes.length);
      others.forEach((route) {
        navigator.push(
            _pageRoute(builder: ((ctx) => screens[route])(context), isMaterial: widget.isMaterial)
              ..popped.then((_) => _silentPop(presenter)));
      });
    }
    presenter.routesSub.onData((newRoutes) {
      final routesDifferentNumber = newRoutes.length - _routes.length;
      if (routesDifferentNumber == 0 && newRoutes.last != _routes.last) {
        navigator.pushReplacement(_pageRoute(
            builder: ((ctx) {
              return screens[newRoutes.last](context);
            }),
            isMaterial: widget.isMaterial)
          ..popped.then((_) => _silentPop(presenter)));
      } else if (routesDifferentNumber > 0) {
        final pushRoutes = newRoutes.sublist(_routes.length, newRoutes.length);
        pushRoutes.forEach((route) {
          navigator.push(
              _pageRoute(builder: ((ctx) => screens[route])(context), isMaterial: widget.isMaterial)
                ..popped.then((_) => _silentPop(presenter)));
        });
      } else if (routesDifferentNumber < 0) {
        _routes.first != newRoutes.first
            ? navigator.pushAndRemoveUntil(
                _pageRoute(
                    builder: ((ctx) {
                      return screens[newRoutes.first](context);
                    }),
                    isMaterial: widget.isMaterial),
                (route) => false)
            : _routes.sublist(newRoutes.length, _routes.length).forEach((route) async {
                navigator.pop();
              });
      }
      _routes.clear();
      _routes.addAll(newRoutes);
    });
  }
}

class OsamTabNavigator<P extends TabNavigationPresenter> extends StatelessWidget {
  final Map<String, Widget> navigators;
  final Map<String, Widget> tabs;
  final bool isMaterial;
  final BoxDecoration tabDecoration;
  final _navigatorKeys = <String, GlobalKey<NavigatorState>>{};

  OsamTabNavigator({Key key, this.navigators, this.tabDecoration, this.tabs, this.isMaterial})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    final presenter = context.presenter<P>();
    if (_navigatorKeys.isEmpty) {
      _navigatorKeys.addAll(navigators.map((k, v) => MapEntry(k, GlobalKey<NavigatorState>())));
    }
    return OsamBuilder<String>(
      key: const ValueKey('tab_ob'),
      stream: presenter.tabStream,
      builder: (ctx, snapshot) {
        return Stack(
          children: <Widget>[
            IndexedStack(
              index: navigators.keys.toList().indexOf(snapshot.data),
              children: navigators.keys.map((tab) {
                return _OsamNavigatorPage(
                  _navigatorKeys[tab],
                  navigators[tab],
                  isMaterial,
                  key: ValueKey(tab.toString() + '_onp'),
                );
              }).toList(),
            ),
            Positioned(
              bottom: 0,
              child: SafeArea(
                bottom: true,
                child: Container(
                  decoration: tabDecoration,
                  padding: const EdgeInsets.symmetric(vertical: 5),
                  constraints: BoxConstraints(
                    maxWidth: MediaQuery.of(context).size.width,
                    minHeight: 48,
                  ),
                  child: Row(
                    children: navigators.keys
                        .map((tab) => Expanded(
                              child: GestureDetector(
                                child: Container(
                                  child: tabs[tab],
                                  alignment: Alignment.center,
                                  color: Colors.transparent,
                                ),
                                onTap: () {
                                  if (snapshot.data != tab) {
                                    presenter.changeTab(tab);
                                  } else {
                                    // TODO(Daniel): add this
//                                    final currentPresenter = navigators[snapshot.data].presenter;
//                                    final routes = currentPresenter.routeStream.value;
//                                    if (routes.length > 1) {
//                                      currentPresenter.pushAndRemoveUntil(routes.first);
//                                    }
                                  }
                                },
                              ),
                            ))
                        .toList(),
                  ),
                ),
              ),
            )
          ],
        );
      },
    );
  }
}

class _OsamNavigatorPage extends StatelessWidget {
  final GlobalKey<NavigatorState> navKey;
  final Widget child;
  final bool isMaterial;

  const _OsamNavigatorPage(this.navKey, this.child, this.isMaterial, {Key key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Navigator(
      key: navKey,
      onGenerateRoute: (RouteSettings settings) => _pageRoute(
          settings: const RouteSettings(isInitialRoute: true),
          builder: (BuildContext context) => child,
          isMaterial: isMaterial),
    );
  }
}

==> 5.2.0.tar.gz: lib/navigation/osam_navigator.dart <==
import 'dart:async';

import 'package:flutter/material.dart';

import '../osam.dart';
import 'navigation_presenter.dart';

PageRoute _pageRoute({WidgetBuilder builder, RouteSettings settings, bool isMaterial = false}) {
  return isMaterial
      ? MaterialPageRoute(builder: builder, settings: settings)
      : CupertinoPageRoute(builder: builder, settings: settings);
}

class OsamNavigator<P extends NavigationPresenter> extends StatefulWidget {
  final Map<String, WidgetBuilder> _screens;
  final bool rootNavigator;
  final bool isMaterial;

  const OsamNavigator(this._screens, {Key key, this.rootNavigator = false, this.isMaterial = false})
      : super(key: key);

  @override
  _OsamNavigatorState createState() => _OsamNavigatorState<P>();
}

class _OsamNavigatorState<P extends NavigationPresenter> extends State<OsamNavigator>
    with AfterLayoutMixin {
  final List<String> _routes = [];
  StreamSubscription<List<String>> routesSub;

  Map<String, WidgetBuilder> get screens => widget._screens;

  @override
  Widget build(BuildContext context) {
    return screens[context.presenter<P>().routesStream.value.first](context);
  }

  void _silentPop(NavigationPresenter presenter) {
    if (presenter.routesStream.value.length > 1) {
      _routes.removeLast();
      presenter.pop();
    }
  }

  @override
  void afterFirstLayout(BuildContext context) {
    final navigator = Navigator.of(context, rootNavigator: widget.rootNavigator);
    final presenter = context.presenter<P>();
    final initialRoutes = presenter.routesStream.value;
    _routes.addAll(initialRoutes);
    if (initialRoutes.length > 1) {
      final firstRoute = initialRoutes.first;
      navigator.pushReplacement(_pageRoute(
          builder: ((ctx) => screens[firstRoute])(context),
          settings: const RouteSettings(isInitialRoute: true),
          isMaterial: widget.isMaterial));
      final others = initialRoutes.sublist(1, initialRoutes.length);
      // await here ?
      others.forEach((route) {
        navigator.push(
            _pageRoute(builder: ((ctx) => screens[route])(context), isMaterial: widget.isMaterial)
              ..popped.then((_) => _silentPop(presenter)));
      });
    }
    presenter.routesSub.onData((newRoutes) {
      final routesDifferentNumber = newRoutes.length - _routes.length;
      if (routesDifferentNumber == 0 && newRoutes.last != _routes.last) {
        navigator.pushReplacement(_pageRoute(
            builder: ((ctx) {
              return screens[newRoutes.last](context);
            }),
            isMaterial: widget.isMaterial)
          ..popped.then((_) => _silentPop(presenter)));
      } else if (routesDifferentNumber > 0) {
        final pushRoutes = newRoutes.sublist(_routes.length, newRoutes.length);
        pushRoutes.forEach((route) {
          navigator.push(
              _pageRoute(builder: ((ctx) => screens[route])(context), isMaterial: widget.isMaterial)
                ..popped.then((_) => _silentPop(presenter)));
        });
      } else if (routesDifferentNumber < 0) {
        _routes.first != newRoutes.first
            ? navigator.pushAndRemoveUntil(
                _pageRoute(
                    builder: ((ctx) {
                      return screens[newRoutes.first](context);
                    }),
                    isMaterial: widget.isMaterial),
                (route) => false)
            : _routes.sublist(newRoutes.length, _routes.length).forEach((route) async {
                navigator.pop();
              });
      }
      _routes.clear();
      _routes.addAll(newRoutes);
    });
  }
}

class OsamTabNavigator<P extends TabNavigationPresenter> extends StatelessWidget {
  final Map<String, Widget> navigators;
  final Map<String, Widget> tabs;
  final bool isMaterial;
  final BoxDecoration tabDecoration;
  final _navigatorKeys = <String, GlobalKey<NavigatorState>>{};

  OsamTabNavigator({Key key, this.navigators, this.tabDecoration, this.tabs, this.isMaterial})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    final presenter = context.presenter<P>();
    if (_navigatorKeys.isEmpty) {
      _navigatorKeys.addAll(navigators.map((k, v) => MapEntry(k, GlobalKey<NavigatorState>())));
    }
    return OsamBuilder<String>(
      key: const ValueKey('tab_ob'),
      stream: presenter.tabStream,
      builder: (ctx, snapshot) {
        return Stack(
          children: <Widget>[
            IndexedStack(
              index: navigators.keys.toList().indexOf(snapshot.data),
              children: navigators.keys.map((tab) {
                return _OsamNavigatorPage(
                  _navigatorKeys[tab],
                  navigators[tab],
                  isMaterial,
                  key: ValueKey(tab.toString() + '_onp'),
                );
              }).toList(),
            ),
            Positioned(
              bottom: 0,
              child: SafeArea(
                bottom: true,
                child: Container(
                  decoration: tabDecoration,
                  padding: const EdgeInsets.symmetric(vertical: 5),
                  constraints: BoxConstraints(
                    maxWidth: MediaQuery.of(context).size.width,
                    minHeight: 48,
                  ),
                  child: Row(
                    children: navigators.keys
                        .map((tab) => Expanded(
                              child: GestureDetector(
                                child: Container(
                                  child: tabs[tab],
                                  alignment: Alignment.center,
                                  color: Colors.transparent,
                                ),
                                onTap: () {
                                  if (snapshot.data != tab) {
                                    presenter.changeTab(tab);
                                  } else {
                                    // TODO(Daniel): add this
//                                    final currentPresenter = navigators[snapshot.data].presenter;
//                                    final routes = currentPresenter.routeStream.value;
//                                    if (routes.length > 1) {
//                                      currentPresenter.pushAndRemoveUntil(routes.first);
//                                    }
                                  }
                                },
                              ),
                            ))
                        .toList(),
                  ),
                ),
              ),
            )
          ],
        );
      },
    );
  }
}

class _OsamNavigatorPage extends StatelessWidget {
  final GlobalKey<NavigatorState> navKey;
  final Widget child;
  final bool isMaterial;

  const _OsamNavigatorPage(this.navKey, this.child, this.isMaterial, {Key key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Navigator(
      key: navKey,
      onGenerateRoute: (RouteSettings settings) => _pageRoute(
          settings: const RouteSettings(isInitialRoute: true),
          builder: (BuildContext context) => child,
          isMaterial: isMaterial),
    );
  }
}
